// initialization
LegoPort=34
call LegoInit(LegoPort)
call LegoRangeInit(4)
LegoDir = 0 // Range sensor points forward
// move forward till a wall is found at 5 inches away
repeat
   call LegoRangeSensor(4,D)
   call LegoForward(SLOW)
until D<5

// turn robot slightly to the right to position parallel to wall
Call LegoDegreesRight(30)
// and establish a new forward orientation
// with range sensor angled left
LegoDir = 1

while true
  call LegoRangeSensor(4,D)
  if D<7 
    call LegoDegreesRight(5)
  elseif D>11
    call LegoDegreesLeft(5)
  else
    call LegoInchForward(5)
  endif
wend
end



//-----------------------------------------------------
sub LegoInit(PortNum)
  _LegoSimulation=False
  // Adjustable parameters
  _FAST = 85
  _SLOW = 78
  _DegTime = 7
  _InchTime = 75
  _LegoDiameter = 11
  _AdvanceTime=750
  _DebugTime=120
  _LegoDebug = false
  Dim SS[3]
  // six directions for forward
  Data wheel1; -1, -1,  0,  1,  1,  0
  Data wheel2;  1,  0, -1, -1,  0,  1
  Data wheel3;  0,  1,  1,  0, -1, -1 
  // easy lefts and easy rights
  Data EL1;  0, -1,  0,  1,  1,  1
  Data EL2;  1,  1,  0, -1,  0,  1
  Data EL3;  0,  1,  1,  1,  0, -1
  Data ER1; -1, -1,  0,  1,  0, -1
  Data ER2;  0, -1, -1, -1,  0,  1
  Data ER3;  0,  1,  0, -1, -1, -1
  _LegoDir = 0
  SetCommPort PortNum,br115200
return

//-----------------------------------------------------
sub LegoForward(mySpeed)
   call LegoMotors(wheel1[_LegoDir]*mySpeed,wheel2[_LegoDir]*mySpeed,wheel3[_LegoDir]*mySpeed)
return

sub LegoBackward(mySpeed)
   call LegoMotors(-wheel1[_LegoDir]*mySpeed,-wheel2[_LegoDir]*mySpeed,-wheel3[_LegoDir]*mySpeed)
return

sub LegoTurn(mySpeed)
   call LegoMotors(-mySpeed,-mySpeed,-mySpeed)
return

sub LegoInchForward(t)
  call LegoForward(_SLOW)
  call Wait(t*_InchTime)
  gosub LegoHalt
return

sub LegoDegreesRight(t)
   call LegoMotors(-_SLOW,-_SLOW,-_SLOW)
   call Wait(t*_DegTime)
   gosub LegoHalt
return

sub LegoDegreesLeft(t)
   call LegoMotors(_SLOW,_SLOW,_SLOW)
   call Wait(t*_DegTime)
   gosub LegoHalt
return

//-----------------------------------------------------
sub LegoMotors(A,B,C)
    //code,code,port,Power,turn on,reg,t ratio,run state,run forever
   SerialOut 12,0,0x80,4,0,A,1,0,0,0x20,0,0,0,0
   SerialOut 12,0,0x80,4,1,B,1,0,0,0x20,0,0,0,0
   SerialOut 12,0,0x80,4,2,C,1,0,0,0x20,0,0,0,0
return
   
 //-----------------------------------------------------
sub Wait(t)
  for i= 1 to t/10
    delay 1
  next
return

//-----------------------------------------------------
sub LegoLineSensor(P,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 3,0,0,7,P-1
   repeat
     CheckSerBuffer NumBytes      
   until NumBytes=18
   serin REPLY
   S = (GetStrByte(REPLY,length(REPLY)) !=2)
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Line Sensor (Port ",P,") = ",S
     input "Press ENTER", A
     LineWidth lw
   endif      
return

//-----------------------------------------------------
sub LegoSoundSensor(P,Sensitivity,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Sound Sensor: Enter 1 ",\
                 "for sound, 0 for no sound."
     input "Type 0 or 1 then Press ENTER", S   
     LineWidth lw
   else
     Sensitivity = Limit(Sensitivity,0,150)
     // read sound sensor three times
     for T2= 0 to 2
       // request a read
       SerialOut 3,0,0,7,P-1
       repeat
         CheckSerBuffer T1 
       until T1=18
       serin d
       SS[T2]=GetStrByte(d,17)
     next
     // determine if there has been a CHANGE in sound level
     Smax=SS[0] \ Smin=SS[0]
     for T2=1 to 2
       if SS[T2]>Smax then Smax=SS[T2]
       if SS[T2]<Smin then Smin=SS[T2]
     next
     S = ((Smax-Smin) > Sensitivity)
   endif  
return

//-----------------------------------------------------
sub LegoBumperSensor(P,&S)
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 3,0,0,7,P-1
   repeat
     CheckSerBuffer T1 
   until T1=18
   serin d
   S =  (GetStrByte(d,15) != 0)
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Bumper Sensor = ",S
     input "Press ENTER", A
     LineWidth lw
   endif
return

//-----------------------------------------------------
sub LegoRangeSensor(P,&S)
   P--
   if _LegoDebug 
     call Wait(_DebugTime)
     gosub LegoHalt
   endif
   SerialOut 7,0,0,0xF,P,2,1,2,0x42
   repeat
      CheckSerBuffer T1 
   until T1=5
   serin d
   SerialOut 3,0,0,0x10,P
   repeat
      CheckSerBuffer T1 
   until T1=22
   serin d
   S=GetStrByte(d,7)
   S = Round(S/2.25)-3
   if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Range Sensor = ", S
     input "Press ENTER", A
     LineWidth lw
   endif
return

//-----------------------------------------------------
sub LegoBumperInit(P)
   //                  port,switch,boolean
   SerialOut 5,0,0x80,5,P-1,1,0x20
   delay 100   
return

//-----------------------------------------------------
sub LegoSoundInit(P)
   //                   port,type,mode
   SerialOut 5,0,0x80,5,P-1,0x07,0
   delay 100
return

//-----------------------------------------------------
sub LegoRangeInit(P)
   //                  port,ultrasonic,raw
   SerialOut 5,0,0x80,5,P-1,0xb,0
   delay 100
   call LegoRangeSensor(P,S)
   call LegoRangeSensor(P,S)
return

//-----------------------------------------------------
sub LegoLineInit(P)
   //                   port,light active,raw
   SerialOut 5,0,0x80,5,P-1,5,0
   delay 100
return

//-----------------------------------------------------
sub Lego3LineInit()
  for i=1 to 3
    call LegoLineInit(i)
  next
return

//-----------------------------------------------------
sub Lego3LineSensors(&P)
  P=0
  if _LegoDebug 
    call Wait(_DebugTime)
    gosub LegoHalt
  endif
  for i=0 to 2      
    SerialOut 3,0,0,7,i
    repeat
      CheckSerBuffer NumBytes      
    until NumBytes=18
    serin REPLY
    if GetStrByte(REPLY,Length(REPLY)) !=2 then P+=(1<< i)
  next
  if _LegoDebug
     GetLineWidth lw \ LineWidth 1
     rectangle 0,550,799,599,black,white
     xyString 50,570,"Line Sensors (BIN) = ",Bin(P,3)
     input "Press ENTER", A
     LineWidth lw
   endif
return  

//-----------------------------------------------------
sub LegoPen(P)
return

//-----------------------------------------------------
LegoAdvance:
  call LegoInchForward(_LegoDiameter/2)
return
  call LegoForward(_SLOW)
  call Wait(_AdvanceTime)
  gosub LegoHalt
return

//-----------------------------------------------------
LegoRetreat:
  call LegoDriveMotors(-_SLOW)
  call Wait(_AdvanceTime)
  gosub LegoHalt
return 

//-----------------------------------------------------
DrawLine:
return 

//-----------------------------------------------------
LegoHalt:
  call LegoForward(0)
return

//-----------------------------------------------------
LegoEasyRight:
  call LegoMotors(ER1[_LegoDir]*_SLOW,ER2[_LegoDir]*_SLOW,ER3[_LegoDir]*_SLOW)
return

 //-----------------------------------------------------
LegoEasyLeft:
  call LegoMotors(EL1[_LegoDir]*_SLOW,EL2[_LegoDir]*_SLOW,EL3[_LegoDir]*_SLOW)
return
//-----------------------------------------------------
